@@ -8,11 +8,13 @@ RCV data models
 from __future__ import annotations
 
 import logging
+from collections.abc import Sequence
 from datetime import date, datetime
+from decimal import Decimal
 from typing import ClassVar, Optional
 
 import pydantic
-from typing_extensions import Self
+from typing_extensions import Self, TypedDict
 
 import cl_sii.dte.data_models
 from cl_sii.base.constants import SII_OFFICIAL_TZ
@@ -101,6 +103,23 @@ class PeriodoTributario:
         )
 
 
+class OtrosImpuestos(TypedDict):
+    codigo_otro_impuesto: Optional[str]
+    """
+    Codigo Otro Imp.
+    """
+
+    valor_otro_impuesto: Optional[int]
+    """
+    Valor Otro Imp.
+    """
+
+    tasa_otro_impuesto: Optional[Decimal]
+    """
+    Tasa Otro Imp.
+    """
+
+
 @pydantic.dataclasses.dataclass(
     frozen=True,
     config=pydantic.ConfigDict(
@@ -283,27 +302,27 @@ class RvDetalleEntry(RcvDetalleEntry):
     Monto IVA
     """
 
-    iva_retenido_total: int
+    iva_retenido_total: Optional[int]
     """
     IVA Retenido Total
     """
 
-    iva_retenido_parcial: int
+    iva_retenido_parcial: Optional[int]
     """
     IVA Retenido Parcial
     """
 
-    iva_no_retenido: int
+    iva_no_retenido: Optional[int]
     """
     IVA no retenido
     """
 
-    iva_propio: int
+    iva_propio: Optional[int]
     """
     IVA propio
     """
 
-    iva_terceros: int
+    iva_terceros: Optional[int]
     """
     IVA Terceros
     """
@@ -413,20 +432,7 @@ class RvDetalleEntry(RcvDetalleEntry):
     NCE o NDE sobre Fact. de Compra
     """
 
-    codigo_otro_imp: Optional[str]
-    """
-    Codigo Otro Imp.
-    """
-
-    valor_otro_imp: Optional[int]
-    """
-    Valor Otro Imp.
-    """
-
-    tasa_otro_imp: Optional[float]
-    """
-    Tasa Otro Imp.
-    """
+    otros_impuestos: Optional[Sequence[OtrosImpuestos]]
 
     ###########################################################################
     # Validators
@@ -536,20 +542,7 @@ class RcDetalleEntry(RcvDetalleEntry):
     NCE o NDE sobre Fact. de Compra
     """
 
-    codigo_otro_impuesto: Optional[str]
-    """
-    Codigo Otro Impuesto
-    """
-
-    valor_otro_impuesto: Optional[int]
-    """
-    Valor Otro Impuesto
-    """
-
-    tasa_otro_impuesto: Optional[float]
-    """
-    Tasa Otro Impuesto
-    """
+    otros_impuestos: Optional[Sequence[OtrosImpuestos]]
 
     ###########################################################################
     # Validators
