@@ -7,10 +7,12 @@ Parse RCV files (CSV)
 
 import csv
 import logging
+from collections.abc import MutableMapping
 from datetime import date, datetime
-from typing import Any, Callable, Dict, Iterable, Optional, Sequence, Tuple, TypeVar
+from typing import Any, Callable, Dict, Iterable, Optional, Sequence, Tuple, TypedDict, TypeVar
 
 import marshmallow
+import marshmallow.experimental.context
 
 from cl_sii.base.constants import SII_OFFICIAL_TZ
 from cl_sii.extras import mm_fields
@@ -98,10 +100,10 @@ def parse_rcv_venta_csv_file(
     """
     # warning: this looks like it would be executed before the iteration begins but it is not.
 
-    schema_context = dict(
-        emisor_rut=rut,
-    )
-    input_csv_row_schema = RcvVentaCsvRowSchema(context=schema_context)
+    schema_context: _RcvVentaCsvRowContext = {
+        'emisor_rut': rut,
+    }
+    input_csv_row_schema = RcvVentaCsvRowSchema()
 
     expected_input_field_names = (
         'Nro',
@@ -153,14 +155,15 @@ def parse_rcv_venta_csv_file(
 
     # note: mypy will complain about returned dataclass type mismatch (and it is right to do so)
     #   but we know from logic which subclass of 'RcvDetalleEntry' will be yielded.
-    yield from _parse_rcv_csv_file(  # type: ignore
-        input_csv_row_schema,
-        expected_input_field_names,
-        fields_to_remove_names,
-        input_file_path,
-        n_rows_offset,
-        max_n_rows,
-    )
+    with _RcvVentaCsvRowSchemaContext(schema_context):
+        yield from _parse_rcv_csv_file(  # type: ignore
+            input_csv_row_schema,
+            expected_input_field_names,
+            fields_to_remove_names,
+            input_file_path,
+            n_rows_offset,
+            max_n_rows,
+        )
 
 
 def parse_rcv_compra_registro_csv_file(
@@ -175,10 +178,10 @@ def parse_rcv_compra_registro_csv_file(
     """
     # warning: this looks like it would be executed before the iteration begins but it is not.
 
-    schema_context = dict(
-        receptor_rut=rut,
-    )
-    input_csv_row_schema = RcvCompraRegistroCsvRowSchema(context=schema_context)
+    schema_context: _RcvCompraCsvRowContext = {
+        'receptor_rut': rut,
+    }
+    input_csv_row_schema = RcvCompraRegistroCsvRowSchema()
 
     expected_input_field_names = (
         'Nro',
@@ -214,14 +217,15 @@ def parse_rcv_compra_registro_csv_file(
 
     # note: mypy will complain about returned dataclass type mismatch (and it is right to do so)
     #   but we know from logic which subclass of 'RcvDetalleEntry' will be yielded.
-    yield from _parse_rcv_csv_file(  # type: ignore
-        input_csv_row_schema,
-        expected_input_field_names,
-        fields_to_remove_names,
-        input_file_path,
-        n_rows_offset,
-        max_n_rows,
-    )
+    with _RcvCompraCsvRowSchemaContext(schema_context):
+        yield from _parse_rcv_csv_file(  # type: ignore
+            input_csv_row_schema,
+            expected_input_field_names,
+            fields_to_remove_names,
+            input_file_path,
+            n_rows_offset,
+            max_n_rows,
+        )
 
 
 def parse_rcv_compra_no_incluir_csv_file(
@@ -236,10 +240,10 @@ def parse_rcv_compra_no_incluir_csv_file(
     """
     # warning: this looks like it would be executed before the iteration begins but it is not.
 
-    schema_context = dict(
-        receptor_rut=rut,
-    )
-    input_csv_row_schema = RcvCompraNoIncluirCsvRowSchema(context=schema_context)
+    schema_context: _RcvCompraCsvRowContext = {
+        'receptor_rut': rut,
+    }
+    input_csv_row_schema = RcvCompraNoIncluirCsvRowSchema()
 
     expected_input_field_names = (
         'Nro',
@@ -272,14 +276,15 @@ def parse_rcv_compra_no_incluir_csv_file(
 
     # note: mypy will complain about returned dataclass type mismatch (and it is right to do so)
     #   but we know from logic which subclass of 'RcvDetalleEntry' will be yielded.
-    yield from _parse_rcv_csv_file(  # type: ignore
-        input_csv_row_schema,
-        expected_input_field_names,
-        fields_to_remove_names,
-        input_file_path,
-        n_rows_offset,
-        max_n_rows,
-    )
+    with _RcvCompraCsvRowSchemaContext(schema_context):
+        yield from _parse_rcv_csv_file(  # type: ignore
+            input_csv_row_schema,
+            expected_input_field_names,
+            fields_to_remove_names,
+            input_file_path,
+            n_rows_offset,
+            max_n_rows,
+        )
 
 
 def parse_rcv_compra_reclamado_csv_file(
@@ -294,10 +299,10 @@ def parse_rcv_compra_reclamado_csv_file(
     """
     # warning: this looks like it would be executed before the iteration begins but it is not.
 
-    schema_context = dict(
-        receptor_rut=rut,
-    )
-    input_csv_row_schema = RcvCompraReclamadoCsvRowSchema(context=schema_context)
+    schema_context: _RcvCompraCsvRowContext = {
+        'receptor_rut': rut,
+    }
+    input_csv_row_schema = RcvCompraReclamadoCsvRowSchema()
 
     expected_input_field_names = (
         'Nro',
@@ -330,14 +335,15 @@ def parse_rcv_compra_reclamado_csv_file(
 
     # note: mypy will complain about returned dataclass type mismatch (and it is right to do so)
     #   but we know from logic which subclass of 'RcvDetalleEntry' will be yielded.
-    yield from _parse_rcv_csv_file(  # type: ignore
-        input_csv_row_schema,
-        expected_input_field_names,
-        fields_to_remove_names,
-        input_file_path,
-        n_rows_offset,
-        max_n_rows,
-    )
+    with _RcvCompraCsvRowSchemaContext(schema_context):
+        yield from _parse_rcv_csv_file(  # type: ignore
+            input_csv_row_schema,
+            expected_input_field_names,
+            fields_to_remove_names,
+            input_file_path,
+            n_rows_offset,
+            max_n_rows,
+        )
 
 
 def parse_rcv_compra_pendiente_csv_file(
@@ -352,10 +358,10 @@ def parse_rcv_compra_pendiente_csv_file(
     """
     # warning: this looks like it would be executed before the iteration begins but it is not.
 
-    schema_context = dict(
-        receptor_rut=rut,
-    )
-    input_csv_row_schema = RcvCompraPendienteCsvRowSchema(context=schema_context)
+    schema_context: _RcvCompraCsvRowContext = {
+        'receptor_rut': rut,
+    }
+    input_csv_row_schema = RcvCompraPendienteCsvRowSchema()
 
     expected_input_field_names = (
         'Nro',
@@ -387,14 +393,15 @@ def parse_rcv_compra_pendiente_csv_file(
 
     # note: mypy will complain about returned dataclass type mismatch (and it is right to do so)
     #   but we know from logic which subclass of 'RcvDetalleEntry' will be yielded.
-    yield from _parse_rcv_csv_file(  # type: ignore
-        input_csv_row_schema,
-        expected_input_field_names,
-        fields_to_remove_names,
-        input_file_path,
-        n_rows_offset,
-        max_n_rows,
-    )
+    with _RcvCompraCsvRowSchemaContext(schema_context):
+        yield from _parse_rcv_csv_file(  # type: ignore
+            input_csv_row_schema,
+            expected_input_field_names,
+            fields_to_remove_names,
+            input_file_path,
+            n_rows_offset,
+            max_n_rows,
+        )
 
 
 ###############################################################################
@@ -416,22 +423,28 @@ class _RcvCsvRowSchemaBase(marshmallow.Schema):
 
     @marshmallow.pre_load
     def preprocess(self, in_data: dict, **kwargs: Any) -> dict:
+        # Convert empty strings to None for fields that allow None.
+        for field_name, field in self.fields.items():
+            data_key = field.data_key if field.data_key is not None else field_name
+            if field.allow_none and data_key in in_data and in_data[data_key] == '':
+                in_data[data_key] = None
+
         # Get required field names from the schema
         required_fields = {
-            field.data_key
+            field.data_key if field.data_key is not None else name
             for name, field in self.fields.items()
             if field.required and field.allow_none is False
         }
         # Remove only required fields that are None or empty string
-        for field in required_fields:
+        for field in required_fields:  # type: ignore[assignment]
             if field in in_data.keys() and (
                 in_data[field] is None or str(in_data[field]).strip() == ''
             ):
                 del in_data[field]
 
         for name, field_item in self.fields.items():
-            data_key = field_item.data_key
-            if data_key is not None and data_key in in_data.keys():
+            data_key = field_item.data_key if field_item.data_key is not None else name
+            if data_key in in_data.keys():
                 if in_data[data_key] == '' or (
                     isinstance(
                         field_item,
@@ -450,6 +463,13 @@ class _RcvCsvRowSchemaBase(marshmallow.Schema):
         return in_data
 
 
+class _RcvVentaCsvRowContext(TypedDict):
+    emisor_rut: Rut
+
+
+_RcvVentaCsvRowSchemaContext = marshmallow.experimental.context.Context[_RcvVentaCsvRowContext]
+
+
 class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
     FIELD_FECHA_RECEPCION_DT_TZ = SII_OFFICIAL_TZ
     FIELD_FECHA_ACUSE_DT_TZ = SII_OFFICIAL_TZ
@@ -519,23 +539,28 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
         data_key='Monto total',
     )
     iva_retenido_total = marshmallow.fields.Integer(
-        required=True,
+        required=False,
+        allow_none=True,
         data_key='IVA Retenido Total',
     )
     iva_retenido_parcial = marshmallow.fields.Integer(
-        required=True,
+        required=False,
+        allow_none=True,
         data_key='IVA Retenido Parcial',
     )
     iva_no_retenido = marshmallow.fields.Integer(
-        required=True,
+        required=False,
+        allow_none=True,
         data_key='IVA no retenido',
     )
     iva_propio = marshmallow.fields.Integer(
-        required=True,
+        required=False,
+        allow_none=True,
         data_key='IVA propio',
     )
     iva_terceros = marshmallow.fields.Integer(
-        required=True,
+        required=False,
+        allow_none=True,
         data_key='IVA Terceros',
     )
     liquidacion_factura_emisor_rut = mm_fields.RutField(
@@ -633,21 +658,29 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
         allow_none=True,
         data_key='NCE o NDE sobre Fact. de Compra',
     )
-    codigo_otro_imp = marshmallow.fields.String(
+
+    otros_impuestos = marshmallow.fields.List(
         required=False,
         allow_none=True,
-        data_key='Codigo Otro Imp.',
-    )
-    valor_otro_imp = marshmallow.fields.Integer(
-        required=False,
-        allow_none=True,
-        data_key='Valor Otro Imp.',
-    )
-    tasa_otro_imp = marshmallow.fields.Decimal(
-        required=False,
-        allow_none=True,
-        data_key='Tasa Otro Imp.',
+        data_key='Otros Impuestos',
+        cls_or_instance=marshmallow.fields.Dict(
+            keys=marshmallow.fields.String(),
+            values=marshmallow.fields.Raw(
+                required=True,
+                allow_none=True,
+            ),
+        ),
     )
+    """
+    Represents the 'Otros Impuestos' group in the CSV as three separate fields:
+    'Codigo Otro Impuesto', 'Valor Otro Impuesto', and 'Tasa Otro Impuesto'.
+    These fields are stored as a list of mappings, each mapping corresponding to a related invoice
+    with the following keys:
+    - Codigo Otro Imp.: String
+    - Valor Otro Imp.: Integer
+    - Tasa Otro Imp.: Decimal
+    """
+
     ###########################################################################
     # fields whose value is set using data passed in the schema context
     ###########################################################################
@@ -663,7 +696,7 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
         #   values of required fields (`required=True`) exist.
 
         # Set field value only if it was not in the input data.
-        in_data.setdefault('emisor_rut', self.context['emisor_rut'])
+        in_data.setdefault('emisor_rut', _RcvVentaCsvRowSchemaContext.get()['emisor_rut'])
 
         # Set tipo_venta from string to enum value.
         if 'Tipo Venta' in in_data:
@@ -684,6 +717,38 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
         if 'RUT Emisor Liquid. Factura' in in_data:
             if in_data['RUT Emisor Liquid. Factura'] in (None, '', '-'):
                 in_data['RUT Emisor Liquid. Factura'] = None
+
+        # Remove Otros Impuestos individual fields if they exist
+        in_data.pop('Codigo Otro Imp.', None)
+        in_data.pop('Valor Otro Imp.', None)
+        in_data.pop('Tasa Otro Imp.', None)
+
+        if 'Otros Impuestos' in in_data and in_data['Otros Impuestos']:
+            otros_impuestos_list = [
+                {
+                    'codigo_otro_impuesto': item.get('codigo_otro_impuesto') or None,
+                    'valor_otro_impuesto': (
+                        item['valor_otro_impuesto']
+                        if item.get('valor_otro_impuesto') not in (None, '', '-')
+                        else None
+                    ),
+                    'tasa_otro_impuesto': (
+                        item['tasa_otro_impuesto']
+                        if item.get('tasa_otro_impuesto') not in (None, '', '-')
+                        else None
+                    ),
+                }
+                for item in in_data['Otros Impuestos']
+                if any(
+                    [
+                        item.get('codigo_otro_impuesto') not in (None, ''),
+                        item.get('valor_otro_impuesto') not in (None, '', '-'),
+                        item.get('tasa_otro_impuesto') not in (None, '', '-'),
+                    ]
+                )
+            ]
+            in_data['Otros Impuestos'] = otros_impuestos_list or None
+
         return in_data
 
     @marshmallow.post_load
@@ -757,9 +822,7 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
             numero_interno = data['numero_interno']
             codigo_sucursal = data['codigo_sucursal']
             nce_o_nde_sobre_factura_de_compra = data['nce_o_nde_sobre_factura_de_compra']
-            codigo_otro_imp = data['codigo_otro_imp']
-            valor_otro_imp = data['valor_otro_imp']
-            tasa_otro_imp = data['tasa_otro_imp']
+            otros_impuestos = data.get('otros_impuestos', None)
         except KeyError as exc:
             raise ValueError("Programming error: a referenced field is missing.") from exc
 
@@ -805,9 +868,7 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
                 numero_interno=numero_interno,
                 codigo_sucursal=codigo_sucursal,
                 nce_o_nde_sobre_factura_de_compra=nce_o_nde_sobre_factura_de_compra,
-                codigo_otro_imp=codigo_otro_imp,
-                valor_otro_imp=valor_otro_imp,
-                tasa_otro_imp=tasa_otro_imp,
+                otros_impuestos=otros_impuestos,
             )
         except (TypeError, ValueError):
             raise
@@ -815,6 +876,13 @@ class RcvVentaCsvRowSchema(_RcvCsvRowSchemaBase):
         return detalle_entry
 
 
+class _RcvCompraCsvRowContext(TypedDict):
+    receptor_rut: Rut
+
+
+_RcvCompraCsvRowSchemaContext = marshmallow.experimental.context.Context[_RcvCompraCsvRowContext]
+
+
 class RcvCompraCsvRowSchema(_RcvCsvRowSchemaBase):
     FIELD_FECHA_RECEPCION_DT_TZ = SII_OFFICIAL_TZ
     FIELD_FECHA_ACUSE_DT_TZ = SII_OFFICIAL_TZ
@@ -853,9 +921,6 @@ class RcvCompraCsvRowSchema(_RcvCsvRowSchemaBase):
         required=True,
         data_key='Razon Social',
     )
-    receptor_rut = mm_fields.RutField(
-        required=True,
-    )
     fecha_recepcion_dt = marshmallow.fields.DateTime(
         format='%d/%m/%Y %H:%M:%S',
         required=True,
@@ -914,21 +979,27 @@ class RcvCompraCsvRowSchema(_RcvCsvRowSchemaBase):
         allow_none=True,
         data_key='NCE o NDE sobre Fact. de Compra',
     )
-    codigo_otro_impuesto = marshmallow.fields.String(
+    otros_impuestos = marshmallow.fields.List(
         required=False,
         allow_none=True,
-        data_key='Codigo Otro Impuesto',
-    )
-    valor_otro_impuesto = marshmallow.fields.Integer(
-        required=False,
-        allow_none=True,
-        data_key='Valor Otro Impuesto',
-    )
-    tasa_otro_impuesto = marshmallow.fields.Decimal(
-        required=False,
-        allow_none=True,
-        data_key='Tasa Otro Impuesto',
+        data_key='Otros Impuestos',
+        cls_or_instance=marshmallow.fields.Dict(
+            keys=marshmallow.fields.String(),
+            values=marshmallow.fields.Raw(
+                required=True,
+                allow_none=True,
+            ),
+        ),
     )
+    """
+    Represents the 'Otros Impuestos' group in the CSV as three separate fields:
+    'Codigo Otro Impuesto', 'Valor Otro Impuesto', and 'Tasa Otro Impuesto'.
+    These fields are stored as a list of mappings, each mapping corresponding to a related invoice
+    with the following keys:
+    - Codigo Otro Impuesto: String
+    - Valor Otro Impuesto: Integer
+    - Tasa Otro Impuesto: Decimal
+    """
 
     ###########################################################################
     # fields whose value is set using data passed in the schema context
@@ -945,7 +1016,7 @@ class RcvCompraCsvRowSchema(_RcvCsvRowSchemaBase):
         #   values of required fields (`required=True`) exist.
 
         # Set field value only if it was not in the input data.
-        in_data.setdefault('receptor_rut', self.context['receptor_rut'])
+        in_data.setdefault('receptor_rut', _RcvCompraCsvRowSchemaContext.get()['receptor_rut'])
 
         # Set tipo_compra from string to enum value.
         if 'Tipo Compra' in in_data:
@@ -964,6 +1035,36 @@ class RcvCompraCsvRowSchema(_RcvCsvRowSchemaBase):
             elif in_data['Tipo Compra'] == 'No Corresp. Incluir':
                 in_data['Tipo Compra'] = RcTipoCompra.NO_CORRESPONDE_INCLUIR.value
 
+        # remove Otros Impuestos individual fields if they exist
+        in_data.pop('Codigo Otro Impuesto', None)
+        in_data.pop('Valor Otro Impuesto', None)
+        in_data.pop('Tasa Otro Impuesto', None)
+
+        if 'Otros Impuestos' in in_data and in_data['Otros Impuestos']:
+            otros_impuestos_list = [
+                {
+                    'codigo_otro_impuesto': item.get('codigo_otro_impuesto') or None,
+                    'valor_otro_impuesto': (
+                        item['valor_otro_impuesto']
+                        if item.get('valor_otro_impuesto') not in (None, '', '-')
+                        else None
+                    ),
+                    'tasa_otro_impuesto': (
+                        item['tasa_otro_impuesto']
+                        if item.get('tasa_otro_impuesto') not in (None, '', '-')
+                        else None
+                    ),
+                }
+                for item in in_data['Otros Impuestos']
+                if any(
+                    [
+                        item.get('codigo_otro_impuesto') not in (None, ''),
+                        item.get('valor_otro_impuesto') not in (None, '', '-'),
+                        item.get('tasa_otro_impuesto') not in (None, '', '-'),
+                    ]
+                )
+            ]
+            in_data['Otros Impuestos'] = otros_impuestos_list or None
         return in_data
 
     @marshmallow.post_load
@@ -1078,9 +1179,7 @@ class RcvCompraRegistroCsvRowSchema(RcvCompraCsvRowSchema):
             nce_o_nde_sobre_factura_de_compra: Optional[str] = data.get(
                 'nce_o_nde_sobre_factura_de_compra'
             )
-            codigo_otro_impuesto: Optional[str] = data.get('codigo_otro_impuesto')
-            valor_otro_impuesto: Optional[int] = data.get('valor_otro_impuesto')
-            tasa_otro_impuesto: Optional[float] = data.get('tasa_otro_impuesto')
+            otros_impuestos = data.get('otros_impuestos', None)
             fecha_acuse_dt: Optional[datetime] = data.get('fecha_acuse_dt')
             tabacos_puros: Optional[int] = data.get('tabacos_puros')
             tabacos_cigarrillos: Optional[int] = data.get('tabacos_cigarrillos')
@@ -1110,9 +1209,7 @@ class RcvCompraRegistroCsvRowSchema(RcvCompraCsvRowSchema):
                 impto_sin_derecho_a_credito=impto_sin_derecho_a_credito,
                 iva_no_retenido=iva_no_retenido,
                 nce_o_nde_sobre_factura_de_compra=nce_o_nde_sobre_factura_de_compra,
-                codigo_otro_impuesto=codigo_otro_impuesto,
-                valor_otro_impuesto=valor_otro_impuesto,
-                tasa_otro_impuesto=tasa_otro_impuesto,
+                otros_impuestos=otros_impuestos,
                 fecha_acuse_dt=fecha_acuse_dt,
                 tabacos_puros=tabacos_puros,
                 tabacos_cigarrillos=tabacos_cigarrillos,
@@ -1198,9 +1295,7 @@ class RcvCompraNoIncluirCsvRowSchema(RcvCompraCsvRowSchema):
             nce_o_nde_sobre_factura_de_compra: Optional[str] = data.get(
                 'nce_o_nde_sobre_factura_de_compra'
             )
-            codigo_otro_impuesto: Optional[str] = data.get('codigo_otro_impuesto')
-            valor_otro_impuesto: Optional[int] = data.get('valor_otro_impuesto')
-            tasa_otro_impuesto: Optional[float] = data.get('tasa_otro_impuesto')
+            otros_impuestos = data.get('otros_impuestos', None)
             fecha_acuse_dt: Optional[datetime] = data.get('fecha_acuse_dt')
         except KeyError as exc:
             raise ValueError("Programming error: a referenced field is missing.") from exc
@@ -1227,9 +1322,7 @@ class RcvCompraNoIncluirCsvRowSchema(RcvCompraCsvRowSchema):
                 impto_sin_derecho_a_credito=impto_sin_derecho_a_credito,
                 iva_no_retenido=iva_no_retenido,
                 nce_o_nde_sobre_factura_de_compra=nce_o_nde_sobre_factura_de_compra,
-                codigo_otro_impuesto=codigo_otro_impuesto,
-                valor_otro_impuesto=valor_otro_impuesto,
-                tasa_otro_impuesto=tasa_otro_impuesto,
+                otros_impuestos=otros_impuestos,
                 fecha_acuse_dt=fecha_acuse_dt,
             )
         except (TypeError, ValueError):
@@ -1319,9 +1412,7 @@ class RcvCompraReclamadoCsvRowSchema(RcvCompraCsvRowSchema):
             nce_o_nde_sobre_factura_de_compra: Optional[str] = data.get(
                 'nce_o_nde_sobre_factura_de_compra'
             )
-            codigo_otro_impuesto: Optional[str] = data.get('codigo_otro_impuesto')
-            valor_otro_impuesto: Optional[int] = data.get('valor_otro_impuesto')
-            tasa_otro_impuesto: Optional[float] = data.get('tasa_otro_impuesto')
+            otros_impuestos = data.get('otros_impuestos')
             fecha_reclamo_dt: Optional[datetime] = data.get('fecha_reclamo_dt')
         except KeyError as exc:
             raise ValueError("Programming error: a referenced field is missing.") from exc
@@ -1348,9 +1439,7 @@ class RcvCompraReclamadoCsvRowSchema(RcvCompraCsvRowSchema):
                 impto_sin_derecho_a_credito=impto_sin_derecho_a_credito,
                 iva_no_retenido=iva_no_retenido,
                 nce_o_nde_sobre_factura_de_compra=nce_o_nde_sobre_factura_de_compra,
-                codigo_otro_impuesto=codigo_otro_impuesto,
-                valor_otro_impuesto=valor_otro_impuesto,
-                tasa_otro_impuesto=tasa_otro_impuesto,
+                otros_impuestos=otros_impuestos,
                 fecha_reclamo_dt=fecha_reclamo_dt,
             )
         except (TypeError, ValueError):
@@ -1404,9 +1493,7 @@ class RcvCompraPendienteCsvRowSchema(RcvCompraCsvRowSchema):
             nce_o_nde_sobre_factura_de_compra: Optional[str] = data.get(
                 'nce_o_nde_sobre_factura_de_compra'
             )
-            codigo_otro_impuesto: Optional[str] = data.get('codigo_otro_impuesto')
-            valor_otro_impuesto: Optional[int] = data.get('valor_otro_impuesto')
-            tasa_otro_impuesto: Optional[float] = data.get('tasa_otro_impuesto')
+            otros_impuestos = data.get('otros_impuestos')
         except KeyError as exc:
             raise ValueError("Programming error: a referenced field is missing.") from exc
 
@@ -1432,9 +1519,7 @@ class RcvCompraPendienteCsvRowSchema(RcvCompraCsvRowSchema):
                 impto_sin_derecho_a_credito=impto_sin_derecho_a_credito,
                 iva_no_retenido=iva_no_retenido,
                 nce_o_nde_sobre_factura_de_compra=nce_o_nde_sobre_factura_de_compra,
-                codigo_otro_impuesto=codigo_otro_impuesto,
-                valor_otro_impuesto=valor_otro_impuesto,
-                tasa_otro_impuesto=tasa_otro_impuesto,
+                otros_impuestos=otros_impuestos,
             )
         except (TypeError, ValueError):
             raise
@@ -1510,35 +1595,116 @@ def _parse_rcv_csv_file(
             expected_field_names=expected_input_field_names,
         )
 
-        g = rows_processing.csv_rows_mm_deserialization_iterator(
-            csv_reader,
-            row_schema=input_csv_row_schema,
-            n_rows_offset=n_rows_offset,
-            max_n_rows=max_n_rows,
-            fields_to_remove_names=fields_to_remove_names,
-        )
+        # Group rows by 'folio + tipo docto + rut' to handle "Otros Impuestos" logic
+        entry_key_groups: MutableMapping[str, Any] = {}
 
-        for row_ix, row_data, deserialized_row_data, validation_errors in g:
-            entry: Optional[RcvDetalleEntry] = None
-            row_errors: Dict[str, object] = {}
-            conversion_error = None
+        # Field names differ slightly between RCV Venta and RCV Compra
+        if isinstance(input_csv_row_schema, RcvVentaCsvRowSchema):
+            codigo_otro_impuesto_key = "Codigo Otro Imp."
+            valor_otro_impuesto_key = "Valor Otro Imp."
+            tasa_otro_impuesto_key = "Tasa Otro Imp."
+            rut_key = "Rut cliente"
+        else:
+            codigo_otro_impuesto_key = "Codigo Otro Impuesto"
+            valor_otro_impuesto_key = "Valor Otro Impuesto"
+            tasa_otro_impuesto_key = "Tasa Otro Impuesto"
+            rut_key = "Rut Proveedor"
 
-            if not validation_errors:
+        # First pass: collect all rows and group by folio + tipo docto + rut
+        for row_ix, row_data in enumerate(csv_reader, start=1):
+            if max_n_rows is not None and row_ix > max_n_rows + n_rows_offset:
+                raise rows_processing.MaxRowsExceeded(f"Exceeded 'max_n_rows' limit: {max_n_rows}.")
+
+            if row_ix <= n_rows_offset:
+                continue
+
+            for _field_name in fields_to_remove_names:
+                row_data.pop(_field_name, None)
+
+            folio = str(row_data.get('Folio')) if row_data.get('Folio') is not None else ''
+            tipo_docto = row_data.get('Tipo Doc')
+            rut = row_data.get(rut_key)
+
+            # Concatenate folio, tipo_docto, and rut to create unique entry key
+            entry_key = f"{folio}_{tipo_docto}_{rut}"
+
+            # If Nro, Fecha Recepcion, and Monto Total are all None or empty,
+            # it's an "otros impuestos" row
+            is_main = not (
+                row_data.get("Nro") in (None, "")
+                and row_data.get("Fecha Recepcion") in (None, "")
+                and (row_data.get("Monto Total") in (None, ""))
+            )
+            otros_impuestos_data = {
+                'codigo_otro_impuesto': (
+                    None
+                    if row_data.get(codigo_otro_impuesto_key) == ''
+                    else row_data.get(codigo_otro_impuesto_key)
+                ),
+                'valor_otro_impuesto': (
+                    None
+                    if row_data.get(valor_otro_impuesto_key) == ''
+                    else row_data.get(valor_otro_impuesto_key)
+                ),
+                'tasa_otro_impuesto': (
+                    None
+                    if row_data.get(tasa_otro_impuesto_key) == ''
+                    else row_data.get(tasa_otro_impuesto_key)
+                ),
+            }
+
+            if entry_key not in entry_key_groups:
+                entry_key_groups[entry_key] = {
+                    'row': (row_ix, row_data),
+                    'otros_impuestos': [otros_impuestos_data],
+                }
+            elif is_main:
+                # Update main row if this is a main entry
+                entry_key_groups[entry_key]['row'] = (row_ix, row_data)
+                entry_key_groups[entry_key]['otros_impuestos'].insert(0, otros_impuestos_data)
+            elif not is_main:
+                if any(otros_impuestos_data.values()):
+                    entry_key_groups[entry_key]['otros_impuestos'].append(otros_impuestos_data)
+
+        # Second pass: yield grouped rows
+        for folio, group in entry_key_groups.items():
+            if group['row']:
+                row_ix, row_data = group['row']
+                row_data['Otros Impuestos'] = group['otros_impuestos']
+
+                # Remove individual "Otros Impuestos" fields
+                row_data.pop(codigo_otro_impuesto_key, None)
+                row_data.pop(tasa_otro_impuesto_key, None)
+                row_data.pop(valor_otro_impuesto_key, None)
+
+                # Deserialize the row
                 try:
-                    entry = input_csv_row_schema.to_detalle_entry(deserialized_row_data)
-                except Exception as exc:
-                    conversion_error = str(exc)
-                    logger.exception(
-                        "Deserialized row data conversion failed for row %d: %s",
-                        row_ix,
-                        conversion_error,
-                        extra={'deserialized_row_data': deserialized_row_data},
-                    )
+                    deserialized_row_data: dict = input_csv_row_schema.load(row_data)
+                    validation_errors: dict = {}
+                except marshmallow.ValidationError as exc:
+                    deserialized_row_data = {}
+                    validation_errors = dict(exc.normalized_messages())
 
-            # Instead of empty dicts, lists, str, etc, we want to have None.
-            if validation_errors:
-                row_errors['validation'] = validation_errors
-            if conversion_error:
-                row_errors['conversion_errors'] = conversion_error
+                entry: Optional[RcvDetalleEntry] = None
+                row_errors: Dict[str, object] = {}
+                conversion_error = None
 
-            yield entry, row_ix, row_data, row_errors
+                if not validation_errors:
+                    try:
+                        entry = input_csv_row_schema.to_detalle_entry(deserialized_row_data)
+                    except Exception as exc:
+                        conversion_error = str(exc)
+                        logger.exception(
+                            "Deserialized row data conversion failed for row %d: %s",
+                            row_ix,
+                            conversion_error,
+                            extra={'deserialized_row_data': deserialized_row_data},
+                        )
+
+                # Instead of empty dicts, lists, str, etc, we want to have None.
+                if validation_errors:
+                    row_errors['validation'] = validation_errors
+                if conversion_error:
+                    row_errors['conversion_errors'] = conversion_error
+
+                yield entry, row_ix, row_data, row_errors
